#pragma vertex vert
#pragma fragment frag
#pragma pc_length 20

struct VertexInput
{
    float2 Position : POSITION;
    float2 UV : TEXCOORD0;
    float4 Color : COLOR;
};

struct VertexOutput
{
    float4 Position : SV_Position;
    float2 UV : TEXCOORD0;
    float4 Color : COLOR;
};

struct PushConstants
{
    float2 Scale;
    float2 Translate;
    uint TextureID;
};

[[vk::combinedImageSampler]][[vk::binding(0)]]
Texture2D fontTexture;

[[vk::combinedImageSampler]][[vk::binding(0)]]
SamplerState fontSampler;

[[vk::push_constant]]
PushConstants pc;

float4 toLinear(float4 sRGB)
{
    bool3 cutoff = sRGB.rgb < (0.04045f);
    float3 higher = pow((sRGB.rgb + (0.055f)) / (1.055f), (2.4f));
    float3 lower = sRGB.rgb / (12.92f);

    return float4(lerp(higher, lower, cutoff), sRGB.a);
}

VertexOutput vert(VertexInput i)
{
    VertexOutput o;
    o.UV = float2(i.UV.x, i.UV.y);
    o.Color = toLinear(i.Color);
    o.Position = float4(i.Position * pc.Scale + pc.Translate, 0.0, 1.0);
    return o;
}

float4 frag(VertexOutput i) : SV_Target
{
    return i.Color * fontTexture.Sample(fontSampler, i.UV);
}